@@ -1,13 +1,15 @@
 """
 Module to process SQL files from edX
 """
+import glob
 import os
 import signal
 import sys
 import traceback
 import zipfile
+import multiprocessing as mp
 from multiprocessing.pool import (
-    Pool as ProcessPool, ThreadPool, TimeoutError
+    Pool as ProcessPool, TimeoutError,
 )
 
 from simeon.download.utilities import (
@@ -42,14 +44,42 @@ def _pool_initializer(fname):
         signal.signal(sig, signal.SIG_DFL)
 
 
+def _sum_batch_sizes(batch):
+    """
+    Get the sum of the file sizes in the given batch of file names
+    """
+    try:
+        return sum(map(lambda f: os.stat(f).st_size, batch))
+    except OSError:
+        return 0
+
+
+def _batch_by_dirs(dirnames, size):
+    """
+    Batch the .gpg files using the directories
+    """
+    bucket = []
+    for dname in dirnames:
+        bucket += glob.iglob(os.path.join(dname, '*.gpg'))
+        bucket += glob.iglob(os.path.join(dname, 'ora', '*.gpg'))
+        if len(bucket) >= size:
+            yield bucket[:]
+            bucket = []
+    if bucket:
+        yield bucket
+
+        
 def _batch_them(items, size):
     """
-    Batch the given items by size
+    Batch the given items by size.
+    If the sum of the file sizes in a batch is greater than
+    500 megabytes, then yield the batch even if it does not
+    have as many as items as the given size parameter.
     """
     bucket = []
     for item in items:
         bucket.append(item)
-        if len(bucket) == size:
+        if len(bucket) >= size:
             yield bucket[:]
             bucket = []
     if bucket:
@@ -74,9 +104,29 @@ def _batch_archive_names(names, size, include_edge=False):
         yield bucket
 
 
+def force_delete_files(files, logger=None):
+    """
+    Delete the given files without regard for whatever or not they exist
+
+    :type files: Iterable[str]
+    :param files: Iterable of file names
+    :type logger: Union[None, logging.Logger]
+    :param logger: A logger object to log messages
+    :rtype: None
+    :return: Returns nothing, but deletes the given files from the local FS
+    """
+    for file_ in files:
+        try:
+            os.unlink(file_)
+        except OSError as excp:
+            if logger is not None:
+                logger.warning(excp)
+            continue
+
+
 def batch_decrypt_files(
     all_files, size=100, verbose=False, logger=None,
-    timeout=None, keepfiles=False,
+    timeout=None, keepfiles=False, njobs=5,
 ):
     """
     Batch the files by the given size and pass each batch to gpg to decrypt.
@@ -93,34 +143,39 @@ def batch_decrypt_files(
     :param timeout: Number of seconds to wait for the decryption to finish
     :type keepfiles: bool
     :param keepfiles: Keep the encrypted files after decrypting them.
+    :type njobs: int
+    :param njobs: Number of threads to use to call gpg in parallel
     :rtype: None
     :return: Nothing, but decrypts the .sql files from the given archive
     """
-    with ThreadPool(5) as pool:
-        results = dict()
-        for batch in _batch_them(all_files, size):
-            async_result = pool.apply_async(
-                    func=decrypt_files, kwds=dict(
-                        fnames=batch, verbose=verbose, logger=logger,
-                        timeout=timeout, keepfiles=keepfiles,
-                    )
+    failures = 0
+    folders = set()
+    orapth = os.path.join('', 'ora', '')
+    for file_ in all_files:
+        if orapth in file_:
+            folders.add(os.path.dirname(os.path.dirname(file_)))
+        else:
+            folders.add(os.path.dirname(file_))
+
+    for batch in _batch_by_dirs(folders, size):
+        try:
+            decrypt_files(
+                fnames=batch, verbose=verbose, logger=logger,
+                timeout=timeout, keepfiles=True
             )
-            results[async_result] = batch
-        failures = 0
-        for result in results:
-            try:
-                result.get()
-            except DecryptionError as excp:
-                failures += 1
-                if logger:
-                    logger.error(excp)
+            if not keepfiles:
+                force_delete_files(batch, logger=logger)
+        except DecryptionError as excp:
+            failures += len(batch)
+            if logger:
+                logger.error(excp)
     if failures:
         raise DecryptionError(
             'Multiple files failed to decrypt. Please consult the logs.'
         )
 
 
-def unpacker(fname, names, ddir, courses=None, tables_only=False):
+def unpacker(fname, names, ddir, cpaths=None, tables_only=False):
     """
     A worker callable to pass a Thread or Process pool
     """
@@ -136,7 +191,7 @@ def unpacker(fname, names, ddir, courses=None, tables_only=False):
             cfolder = os.path.basename(os.path.dirname(target_dir))
         else:
             cfolder = os.path.basename(target_dir)
-        if courses and cfolder not in courses:
+        if cpaths and cfolder not in cpaths:
             continue
         if 'ccx' in cfolder:
             dir_segments = cfolder.replace('-', '__', 1).split('-')
@@ -156,8 +211,11 @@ def unpacker(fname, names, ddir, courses=None, tables_only=False):
             continue
         os.makedirs(target_dir, exist_ok=True)
         with proc_zfile.open(name) as zh, open(target_name, 'wb') as fh:
-            for line in zh:
-                fh.write(line)
+            while True:
+                chunk = zh.read(1048576)
+                if not chunk:
+                    break
+                fh.write(chunk)
         targets.append(target_name)
     return targets
 
@@ -183,15 +241,15 @@ def process_sql_archive(
     :param tables_only: Whether to extract file names only (no unarchiving)
     :type debug: bool
     :param debug: Show the stacktrace when an error occurs
-    :rtype: Iterable[str]
-    :return: List of file names
+    :rtype: Set[str]
+    :return: A set of file names
     """
     cpaths = set()
     for c in (courses or []):
         cpaths.add(c.replace('/', '-'))
     if ddir is None:
         ddir, _ = os.path.split(archive)
-    out = []
+    out = set()
     with zipfile.ZipFile(archive) as zf:
         names = zf.namelist()
     batches = _batch_archive_names(names, len(names) // size, include_edge)
@@ -210,7 +268,7 @@ def process_sql_archive(
                 try:
                     targets = result.get(timeout=60)
                     processed += 1
-                    out.extend(targets or [])
+                    out.update(targets or [])
                 except TimeoutError:
                     continue
                 except KeyboardInterrupt:
