@@ -52,6 +52,21 @@ def _extract_values(record, paths):
         yield subrec.get(end, '')
 
 
+def _is_gpg_legacy():
+    """
+    Check if gpg is of a version below 2.1
+    """
+    p = sb.Popen(shlex.split('gpg --version'), stdout=sb.PIPE, stderr=sb.PIPE)
+    if p.wait() != 0:
+        raise DecryptionError(
+            'The gpg command does not exist, '
+            'or it is not properly configured.'
+        )
+    line = p.stdout.readline().decode('utf8', 'ignore').strip()
+    version = line.split()[-1]
+    return version < '2.1.0'
+
+
 def decrypt_files(
     fnames, verbose=True, logger=None, timeout=None, keepfiles=False
 ):
@@ -76,28 +91,38 @@ def decrypt_files(
     """
     if isinstance(fnames, str):
         fnames = [fnames]
-    verbosity = '--verbose' if verbose else ''
-    cmd = 'gpg {v} --batch --yes --decrypt-files {f}'.format(
-        f=' '.join(fnames), v=verbosity
+    cmd = (
+        'gpg {v}--status-fd 2 --batch --yes {p}'
+        '--decrypt-files {f}'
     )
+    verbosity = '--verbose ' if verbose else ''
+    pinentry = '' if _is_gpg_legacy() else '--pinentry error '
+    cmd = cmd.format(f=' '.join(fnames), v=verbosity, p=pinentry)
     if verbose and logger is not None:
         logger.info('{m}...'.format(m=cmd[:200]))
     proc =  sb.Popen(shlex.split(cmd), stdout=sb.PIPE, stderr=sb.PIPE)
     if proc.wait(timeout=timeout) != 0:
-        err = proc.stderr.read().decode('utf8', 'ignore').strip()
-        msg = 'Failed to decrypt file names {f} with return code {rc}: {e}'
-        raise DecryptionError(
-            msg.format(f=' '.join(fnames), e=err, rc=proc.returncode)
-        )
-    if verbose and logger is not None:
-        for line in proc.stdout:
-            logger.info(line.decode('utf8', 'ignore').strip())
+        errs = []
+        for line in proc.stderr:
+            errs.append(line.decode('utf8', 'ignore').strip())
+        msg = 'Failed to decrypt file names {f} with return code {rc}:\n{e}'
+        raise DecryptionError(msg.format(f=' '.join(
+            fnames), e='\n'.join(errs), rc=proc.returncode
+        ))
     if not keepfiles:
         for file_ in fnames:
             try:
-                os.remove(file_)
-            except:
-                continue
+                os.unlink(file_)
+            except OSError as excp:
+                if logger:
+                    msg = 'Failed to delete encrypted file {f}: {e}'
+                    logger.warning(msg.format(f=file_, e=excp))
+    if verbose:
+        msgs = []
+        for line in proc.stdout:
+            msgs.append(line.decode('utf8', 'ignore').strip())
+        if msgs:
+            logger.warning('\n'.join(msgs))
     return True
 
 
@@ -154,7 +179,7 @@ def get_sql_course_id(course_str: str) -> str:
     return course_str.split(':')[-1].replace('+', '/')
 
 
-def format_sql_filename(fname: str) -> (str, str):
+def format_sql_filename(fname: str):
     """
     Reformat the given edX SQL encrypted file name into a name indicative
     of where the file should end up after the SQL archive is unpacked.
@@ -183,15 +208,7 @@ def format_sql_filename(fname: str) -> (str, str):
     out = '{o}-{e}.gpg'.format(o=out, e=ending) if ending else out
     if 'ora/' in fname:
         out = os.path.join('ora', out)
-    return (
-        fname,
-        os.path.join(
-            site, dirname,
-            cid,
-            # cid.replace('-', '__', 2).replace('-', '_').replace('.', '_'),
-            out,
-        )
-    )
+    return fname, os.path.join(site, dirname, cid, out)
 
 
 def get_course_id(record: dict, paths=COURSE_PATHS) -> str:
