@@ -4,6 +4,7 @@ Utility functions for the simeon CLI tool
 import glob
 import logging
 import os
+import socket
 import sys
 import configparser
 from argparse import ArgumentTypeError
@@ -334,16 +335,20 @@ def make_logger(user='SIMEON', verbose=True, stream=None):
         stream = open(stream, 'a')
     level = logging.INFO if verbose else logging.WARN
     formatter = logging.Formatter(
-        '%(asctime)s:%(levelname)s:%(name)s:%(message)s',
+        '%(asctime)s:%(hostname)s:%(levelname)s:%(name)s:%(message)s',
         '%Y-%m-%d %H:%M:%S%z'
     )
+    # formatter = logging.Formatter(
+    #     '%(asctime)s:%(levelname)s:%(name)s:%(message)s',
+    #     '%Y-%m-%d %H:%M:%S%z'
+    # )
     handler = logging.StreamHandler(stream=stream)
     handler.setLevel(level)
     handler.set_name(user)
     handler.setFormatter(formatter)
     logger = logging.Logger(user, level)
     logger.addHandler(handler)
-    return logger
+    return logging.LoggerAdapter(logger, {'hostname': socket.gethostname()})
 
 
 def make_config_file(output=None):
@@ -470,6 +475,7 @@ def course_paths_from_file(fname):
     out = set()
     with open(fname) as fh:
         for line in fh:
+            line = line.split(':')[-1]
             c = line.strip().replace('/', '__').replace('+', '__')
             out.add(c.replace('.', '_').replace('-', '_').lower())
         return out
@@ -587,8 +593,34 @@ def process_extra_args(extras):
     return out
 
 
+class NumberRange(object):
+    """
+    Check if a number is within a range. Otherwise, raise ArgumentTypeError
+    """
+    def __init__(self, type_=int, lower=1, upper=50):
+        self.type_ = type_
+        self.lower = lower
+        self.upper = upper
+
+    def __call__(self, value):
+        try:
+            v = self.type_(value)
+        except Exception as excp:
+            msg = (
+                'The given value {v} could not be converted to a number. '
+                'Please provide a valid value: {e}.'
+            )
+            raise ArgumentTypeError(msg.format(v=value, e=excp)) from None
+        if not (self.lower <= v <= self.upper):
+            msg = '{v} is not in the range [{l}, {u}]'
+            raise ArgumentTypeError(
+                msg.format(v=value, l=self.lower, u=self.upper)
+            )
+        return v
+
+
 __all__ = [
-    'bq_table', 'course_listings', 'course_paths_from_file',
+    'NumberRange', 'bq_table', 'course_listings', 'course_paths_from_file',
     'courses_from_file', 'expand_paths', 'filter_generated_items',
     'find_config', 'gcs_bucket', 'is_parallel', 'items_from_files',
     'make_config_file', 'make_logger', 'optional_file',
